#include "greeterhandler.h"

#include "kdm_greet.h"
#include "utils.h"
#include "kgreeterplugin.h"
#include "kdmshutdown.h"
#include "exceptions.h"


GreeterHandler::GreeterHandler(const int dmSocket, QObject* parent) :
    QObject(parent),
    m_socketNotifier(dmSocket, QSocketNotifier::Read)
{
    stop();

    connect(&m_socketNotifier, &QSocketNotifier::activated,
            this, &GreeterHandler::handleSocket);
}

GreeterHandler::~GreeterHandler()
{
}

void GreeterHandler::start()
{
    m_socketNotifier.setEnabled(true);
}

void GreeterHandler::stop()
{
    m_socketNotifier.setEnabled(false);
}

const QString GreeterHandler::readLastUserSession(const QString& username)
{
    gSendInt(G_ReadDmrc);
    gSendStr(username.toUtf8());
    gRecvInt(); // ignore status code ...

    gSendInt(G_GetDmrc);
    gSendStr("Session");

    const auto sessionChar = gRecvStr();
    const auto session = QString::fromUtf8(sessionChar);
    free(sessionChar);

    return session;
}

const DpyDescriptions GreeterHandler::getDisplaySessions(const int flags)
{
    DpyDescriptions sessions;

    foreach (const auto& session, fetchSessions(flags)) {
        QString user, loc;
        decodeSession(session, user, loc);

        DpyDescription description;
        description.display = session.display;
        description.from = session.from;
        description.user = session.user;
        description.session = session.session;
        description.vt = session.vt;
        description.flags = session.flags;
        description.count = session.count;
        description.decodeLoc = loc;
        description.decodeUser = user;
        description.isCurrentSession = session.flags & isSelf;

        sessions << description;
    }

    return sessions;
}

void GreeterHandler::switchDisplay(const int vt)
{
    gSet(1);
    gSendInt(G_Activate);
    gSendInt(vt);
    gSet(0);
}

void GreeterHandler::handleLoginRequest(const AuthData& data)
{
    gSendInt(G_Verify);
    gSendStr("classic");

    gSendStr(data.username.toLocal8Bit());
    gSendInt(KGreeterPluginHandler::IsUser);

    gSendStr(data.password.toLocal8Bit());
    gSendInt(KGreeterPluginHandler::IsPassword | KGreeterPluginHandler::IsSecret);
}

void GreeterHandler::handleSocket()
{
    debug("handleVerify ...\n");

    int ret = gRecvInt();

    switch (ret) {
    case V_GET_TEXT: {
        debug(" V_GET_TEXT\n");
        const auto msg = gRecvStr();
        debug("  prompt %\"s\n", msg);
        const auto echo = gRecvInt();
        debug("  echo = %d\n", echo);
        const auto ndelay = gRecvInt();
        debug("  ndelay = %d\n", ndelay);
        free(msg);
        break;
    }
    case V_GET_BINARY: {
        debug(" V_GET_BINARY\n");
        const auto msg = gRecvArr(&ret);
        debug("  prompt %\"s\n", msg);
        debug("  %d bytes prompt\n", ret);
        const auto ndelay = gRecvInt();
        debug("  ndelay = %d\n", ndelay);
        free(msg);
        break;
    }
    case V_PUT_USER: {
        debug(" V_PUT_USER\n");
        const auto msg = gRecvStr();
        debug("  prompt %\"s\n", msg);
        free(msg);
        break;
    }
    case V_PRE_OK: {
        debug(" V_PRE_OK\n");
        break;
    }
    case V_MSG_ERR: {
        debug(" V_MSG_ERR\n");
        const auto msg = gRecvStr();
        debug("  prompt %\"s\n", msg);
        free(msg);
        gSendInt(0);
        break;
    }
    case V_MSG_INFO_AUTH: {
        debug(" V_MSG_INFO_AUTH\n");
        const auto msg = gRecvStr();
        debug("  prompt %\"s\n", msg);
        free(msg);
        gSendInt(0);
        break;
    }
    case V_MSG_INFO: {
        debug(" V_MSG_INFO\n");
        const auto msg = gRecvStr();
        debug("  display %\"s\n", msg);
        free(msg);
        gSendInt(0);
        return;
    }
    case V_CHTOK_AUTH: {
        debug(" V_CHTOK_AUTH\n");
        emit successLogin();
        break;
    }
    case V_CHTOK: {
        debug(" V_CHTOK\n");
        emit successLogin();
        break;
    }
    case V_OK: {
        debug(" V_OK\n");
        emit successLogin();
        break;
    }
    case V_AUTH: {
        debug(" V_AUTH\n");
        emit loginFailed();
        break;
    }
    case V_FAIL: {
        debug(" V_FAIL\n");
        break;
    }
    default:
        throw CommonException(tr("Unknown V_xxx code '%1' from core").arg(ret));
    }
}

void GreeterHandler::doShutdown(const int type, const QString &os)
{
    gSet(1);
    gSendInt(G_Shutdown);
    gSendInt(type);
    gSendInt(0);
    gSendInt(0);
    gSendInt(SHUT_FORCE);
    gSendInt(0);
    gSendStr(os.toUtf8().data());
    gSet(0);
}

void GreeterHandler::setSessionType(const QString& session)
{
    gSendInt(G_PutDmrc);
    gSendStr("Session");
    gSendStr(session.toUtf8());
}

void GreeterHandler::handleConsoleLoginRequest()
{
    gSet(1);
    gSendInt(G_Console);
    gSet(0);
}
